/*
 * Copyright (c) 2016, Oracle and/or its affiliates. All rights reserved.
 * ORACLE PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.sun.ts.tests.jsonp.api.mergetests;

import com.sun.ts.lib.util.TestUtil;
import com.sun.ts.tests.jsonp.api.common.TestResult;
import javax.json.JsonObject;

import static com.sun.ts.tests.jsonp.api.common.SimpleValues.*;

// $Id: MergeAddValue.java 74744 2016-12-05 15:09:04Z tkraus $
/**
 * RFC 7396: JavaScript Object Notation (JSON) Merge Patch compatibility tests.<br>
 * Checks scenario described in
 * {@see <a href="https://tools.ietf.org/html/rfc7396#section-1">RFC 7396: 1. Introduction</a>}:
 * If the provided merge patch contains members that do not appear within the target, those members are added.
 */
public class MergeAddValue extends MergeCommon {
    
    /**
     * Creates an instance of RFC 7396 value adding test.
     */
    MergeAddValue() {
        super();
    }

    /**
     * Test RFC 7396: Adding non existing values. Suite entry point.
     * @return Result of all tests in this suite.
     */
    TestResult test() {
        final TestResult result = new TestResult("RFC 7396: Add non existing values");
        TestUtil.logMsg("Testing RFC 7396: Add non existing values");
        testStringOnEmptyObject(result);
        testStringOnsimpleObject(result);
        testIntOnEmptyObject(result);
        testIntOnsimpleObject(result);
        testBoolOnEmptyObject(result);
        testBoolOnsimpleObject(result);
        testObjectOnEmptyObject(result);
        testObjectOnsimpleObject(result);
        return result;
    }

    /**
     * Test RFC 7396 patch and diff for {@code String} on empty JSON object.
     * @param result Tests result record.
     */
    private void testStringOnEmptyObject(final TestResult result) {
        TestUtil.logMsg(" - for String on empty JSON object");
        final JsonObject in = createEmptyObject();
        final JsonObject patch = createSimpleObjectStr();
        final JsonObject check = createSimpleObjectStr();
        simpleMerge(result, in, patch, check);
        simpleDiff(result, in, check, patch);
    }

    /**
     * Test RFC 7396 patch and diff for {@code String} on simple JSON object.
     * @param result Tests result record.
     */
    private void testStringOnsimpleObject(final TestResult result) {
        TestUtil.logMsg(" - for String on simple JSON object");
        final JsonObject in = createSimpleObject();
        final JsonObject patch = createSimpleObjectStr();
        final JsonObject check = createSimpleObjectWithStr();
        simpleMerge(result, in, patch, check);
        simpleDiff(result, in, check, patch);
    }

    /**
     * Test RFC 7396 patch and diff for {@code int} on empty JSON object.
     * @param result Tests result record.
     */
    private void testIntOnEmptyObject(final TestResult result) {
        TestUtil.logMsg(" - for int on empty JSON object");
        final JsonObject in = createEmptyObject();
        final JsonObject patch = createSimpleObjectInt();
        final JsonObject check = createSimpleObjectInt();
        simpleMerge(result, in, patch, check);
        simpleDiff(result, in, check, patch);
    }

    /**
     * Test RFC 7396 patch and diff for {@code int} on simple JSON object.
     * @param result Tests result record.
     */
    private void testIntOnsimpleObject(final TestResult result) {
        TestUtil.logMsg(" - for int on simple JSON object");
        final JsonObject in = createSimpleObject();
        final JsonObject patch = createSimpleObjectInt();
        final JsonObject check = createSimpleObjectWithInt();
        simpleMerge(result, in, patch, check);
        simpleDiff(result, in, check, patch);
    }

    /**
     * Test RFC 7396 patch and diff for {@code boolean} on empty JSON object.
     * @param result Tests result record.
     */
    private void testBoolOnEmptyObject(final TestResult result) {
        TestUtil.logMsg(" - for boolean on empty JSON object");
        final JsonObject in = createEmptyObject();
        final JsonObject patch = createSimpleObjectBool();
        final JsonObject check = createSimpleObjectBool();
        simpleMerge(result, in, patch, check);
        simpleDiff(result, in, check, patch);
    }

    /**
     * Test RFC 7396 patch and diff for {@code boolean} on simple JSON object.
     * @param result Tests result record.
     */
    private void testBoolOnsimpleObject(final TestResult result) {
        TestUtil.logMsg(" - for boolean on simple JSON object");
        final JsonObject in = createSimpleObject();
        final JsonObject patch = createSimpleObjectBool();
        final JsonObject check = createSimpleObjectWithBool();
        simpleMerge(result, in, patch, check);
        simpleDiff(result, in, check, patch);
    }

    /**
     * Test RFC 7396 patch and diff for {@code JsonObject} on empty JSON object.
     * @param result Tests result record.
     */
    private void testObjectOnEmptyObject(final TestResult result) {
        TestUtil.logMsg(" - for JsonObject on empty JSON object");
        final JsonObject in = createEmptyObject();
        final JsonObject patch = createSimpleObjectObject();
        final JsonObject check = createSimpleObjectObject();
        simpleMerge(result, in, patch, check);
        simpleDiff(result, in, check, patch);
    }

    /**
     * Test RFC 7396 patch and diff for {@code JsonObject} on compound JSON object.
     * @param result Tests result record.
     */
    private void testObjectOnsimpleObject(final TestResult result) {
        TestUtil.logMsg(" - for JsonObject on compound JSON object");
        final JsonObject in = createCompoundObject();
        final JsonObject patch = createSimpleObjectObject();
        final JsonObject check = createCompoundObjectWithObject();
        simpleMerge(result, in, patch, check);
        simpleDiff(result, in, check, patch);
    }

}
